﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
 
/*
 * In this demo, the value that you enter into the 1st textBox on Form1 will manifest to a table on iPhone. 
 * The item you tap on iPhone is sent to the 2nd textBox.
 * 
 * 1. Download iQuipment iPhone app if you have not done so. 
 * 2. Enter the following url to the iQuipment "http://yourComputerIP/iphone/xml/demo1.xml". Do not include the quoation. 
 * 3. You should see the table "Apple, Orange, Banana" on your iPhone.
 * 4. Tap one of the items and see it appears on the 2nd textbox of Form1. 
 * */

namespace Demo
{
    public partial class Form1 : Form
    {
        //locks
        object setDataLock = new object();
        object getDataLock = new object();
        object deleteDataLock = new object();

        const string myViewName = "my dynamic section";
        const string dataName_rowArray = "row array";
        const string dataName_selectedItem = "selected item";
        const string rowArray_init="Apple;Orange;Plum";
        internal class DeviceData
        {
            public string rowArray = rowArray_init;
            public string selectedData="Apple";
        }

        Dictionary<string, DeviceData> deviceDataDictionary = new Dictionary<string, DeviceData>(); //<IPAddress, DeviceData>, used to store data for each devices
        public Form1()
        {

            InitializeComponent();

            //1. Expose all the variables at the beginning simply by assigning a new name by null or a value--------------------
            //exposed and update variable dataName_rowArray (for showing the table rows onon iOS device. 
            INFOMATO.WCF.DataExchangeService.servicePostData(myViewName, dataName_rowArray, "");
            //expose variable name selected_item (for showing the item that you tap on Apple's i-Devices)
            INFOMATO.WCF.DataExchangeService.servicePostData(myViewName, dataName_selectedItem, "");

            //2. Subscribe to the event when a tap action takes place onon iOS device. 
            INFOMATO.WCF.DataExchangeService.clientSetData_ID_Event +=
                new INFOMATO.WCF.DataExchangeService.ClientSetData_ID_Delegate(DataExchangeService_clientSetData_ID_Event); //event triggered by client set data
            //3. Subscribe to the event when a delete action takes place on iOS device.
            INFOMATO.WCF.DataExchangeService.clientDeleteData_ID_Event += new INFOMATO.WCF.DataExchangeService.ClientDeleteData_ID_Delegate(DataExchangeService_clientDeleteData_ID_Event);
            textBox_rowArray.Text = rowArray_init; //initialize the textBox1. 
            INFOMATO.WCF.DataExchangeService.clientGetData_ID_Event += new INFOMATO.WCF.DataExchangeService.ClientGetData_ID_Delegate(DataExchangeService_clientGetData_ID_Event);
        }

        void DataExchangeService_clientGetData_ID_Event(string clientIP, string viewName, string dataName, long processID)
        {
            lock (getDataLock)
            {
                DeviceData theDeviceData = getDeviceData(clientIP); //get the device data for this client
                if (dataName == dataName_rowArray)
                {
                    INFOMATO.WCF.DataExchangeService.servicePostData(clientIP, myViewName, dataName, theDeviceData.rowArray);
                    UpdateTextGUIObject(textBox_rowArray, theDeviceData.rowArray); //the incoming thread may not be the GUI thread, call this function takes care of this issue. 
                }
                else if (dataName == dataName_selectedItem)
                {
                    INFOMATO.WCF.DataExchangeService.servicePostData(clientIP, myViewName, dataName, theDeviceData.selectedData);
                    UpdateTextGUIObject(textBox_selectedItem, theDeviceData.selectedData); //the incoming thread may not be the GUI thread, call this function takes care of this issue. 

                }
            }

        }

        //event handler for the INFOMATO.WCF.DataExchangeService.clientSetData_ID_Event 
        void DataExchangeService_clientSetData_ID_Event(string clientIP, string viewName, string dataName, object dataValue, long processID)
        {
            lock (setDataLock)
            {
                DeviceData theDeviceData = getDeviceData(clientIP);
                if (viewName == myViewName && dataName == dataName_selectedItem)
                {
                    theDeviceData.selectedData = (string)dataValue;
                    UpdateTextGUIObject(textBox_selectedItem, (string)dataValue); //the incoming thread may not be the GUI thread, call this function takes care of this issue. 
                }

            }
        }

        //event handller for INFOMATO.WCF.DataExchangeService.clientSetData_ID_Event
        void DataExchangeService_clientDeleteData_ID_Event(string clientIP, string viewName, string dataName, object dataValue, long processID)
        {
            lock (deleteDataLock)
            {
                DeviceData theDeviceData = getDeviceData(clientIP); //get the device data for this client

                if (dataValue != null && ((string)dataValue).Length > 0 && dataName == dataName_rowArray)
                {
                    //remove the selected string from textBox1.text

                    int startIndex = theDeviceData.rowArray.IndexOf((string)dataValue);
                    if (startIndex > -1)
                    {
                        int endIndex = theDeviceData.rowArray.IndexOf(';', startIndex);
                        if (endIndex < 0) endIndex = theDeviceData.rowArray.Length - 1;
                        string newRowArray = theDeviceData.rowArray.Remove(startIndex, endIndex - startIndex + 1).Trim().TrimEnd(new char[] { ';' });
                        theDeviceData.rowArray = newRowArray;
                        INFOMATO.WCF.DataExchangeService.servicePostData(clientIP, dataName_rowArray, newRowArray);
                        UpdateTextGUIObject(textBox_rowArray, newRowArray); //UI thread safe update textBox1 value

                    }
                    else
                    {
                        UpdateTextGUIObject(textBox_rowArray, theDeviceData.rowArray);
                    }
                }
            }
        }


        //UIThread safe procedure to update UI parameters
        private void UpdateTextGUIObject(object guiObject, string stringValue)
        {
            //check it the thread is a UI thread
            if (this.InvokeRequired) //not UI thread
            {
                MethodInvoker del = delegate
                {
                    UpdateTextGUIObject(guiObject, stringValue);
                };
                this.Invoke(del);
         
            }
            else //Yes, it is UI thread
            {
                if (guiObject is TextBox)
                {
                    ((TextBox)guiObject).Text = stringValue;
                }
                else if (guiObject is RichTextBox)
                {
                    ((RichTextBox)guiObject).Text = stringValue;
                }
                else if (guiObject is ComboBox && !String.IsNullOrEmpty( stringValue))
                {
                    bool cotainsIP = false;
                    //see if the comobox already contains this IP
                    foreach (object item in comboBox_ip.Items)
                    {
                        //if the IP already exists in the comboBox
                        if (item.ToString() == stringValue)
                        {
                            cotainsIP = true;
                            comboBox_ip.SelectedItem = item;
                            return;
                        }
                    }

                    //IP does not exisits in the comboBox; add the new IP to comboBox and select it
                    if(!cotainsIP)
                    {
                        object newItem=comboBox_ip.Items.Add(stringValue);
                        comboBox_ip.SelectedItem = newItem;
                    }

                }
            }
        }


        //event handler for the textbox1 when its value changes
        private void textBox1_TextChanged(object sender, EventArgs e)
        {
            //INFOMATO.WCF.DataExchangeService.servicePostData(myViewName, dataName_rowArray, textBox1.Text); //exposed and update variable dataName_rowArray;
        }

        //event handler for the textbox2 when its value changes.
        private void textBox2_TextChanged(object sender, EventArgs e)
        {

        }

        private DeviceData getDeviceData(string IPAddress)
        {
            if (deviceDataDictionary.ContainsKey(IPAddress))
            {
                UpdateTextGUIObject(comboBox_ip, IPAddress);
                return deviceDataDictionary[IPAddress];
            }
            else
            {
                //create data object for the newly connected device
                DeviceData newData = new DeviceData();
                deviceDataDictionary.Add(IPAddress, newData);
                UpdateTextGUIObject(comboBox_ip, IPAddress); //add the connected IP to the combo box
                return newData;
            }
        }

        //show the data selected.
        private void comboBox_ip_SelectedIndexChanged(object sender, EventArgs e)
        {
            DeviceData data = getDeviceData(comboBox_ip.SelectedItem.ToString());
            textBox_rowArray.Text= data.rowArray;
            textBox_selectedItem.Text = data.selectedData;
        }
    }
}
